//
//  TriggerFragment.cc   (W.Badgett)
//

#include "TriggerFragment.h"
#include "LariatFragment.h"
#include <stdexcept>

// Constructor
TriggerFragment::TriggerFragment()
{
  triggerFragment.fragmentType = LariatFragment::FRAGMENT_TYPE_TRIGGER_FRAGMENT;
  triggerFragment.fragmentSize = sizeof(TriggerFragment_t);
  triggerFragment.nEntries = 0;
  triggerFragment.patterns = NULL;
}

uint32_t TriggerFragment::streamToBuffer(std::ostream& output)
{
  uint32_t nBytes=0;

  uint32_t headerSize = sizeof(TriggerFragment_t) - 
    sizeof(TriggerPattern::TriggerPattern_t *);

  uint32_t bodySize = 
    triggerFragment.nEntries * sizeof(TriggerPattern::TriggerPattern_t);

  triggerFragment.fragmentSize = headerSize + bodySize;

  output.write((char *)&triggerFragment,headerSize);
  nBytes += headerSize;

  output.write((char *)triggerFragment.patterns,bodySize);
  nBytes += bodySize;

  return nBytes;
}


uint32_t TriggerFragment::fillFromBuffer(std::istream& input)
{
  uint32_t nBytes=0;

  uint32_t headerSize = sizeof(TriggerFragment_t) - 
    sizeof(TriggerPattern::TriggerPattern_t *);

  input.read((char *)&triggerFragment,headerSize);
  nBytes += headerSize;

  uint32_t bodySize = 
    triggerFragment.nEntries * sizeof(TriggerPattern::TriggerPattern_t);

  if ( triggerFragment.patterns != NULL )
  { 
    free(triggerFragment.patterns); 
    triggerFragment.patterns = NULL;
  }

  triggerFragment.patterns = (TriggerPattern::TriggerPattern_t *)
    malloc(bodySize);

  input.read((char *)triggerFragment.patterns,bodySize);
  nBytes += bodySize;

  return nBytes;
}

void TriggerFragment::print(std::ostream & os)
{
  char line[256];
  os << "TriggerFragment" << std::endl;
  os << "  FragmentSize     " << triggerFragment.fragmentSize << std::endl;
  os << "  FragmentType     " << triggerFragment.fragmentType << std::endl;
  os << "  NEntries         " << triggerFragment.nEntries << std::endl;
  os << "   n    TimeStamp   Pattern" << std::endl;
  for ( uint32_t i=0; i<triggerFragment.nEntries; i++)
  {
    sprintf(line, "%4d: ", (int)i);
    os << line;
    TriggerPattern::printTriggerPattern(triggerFragment.patterns[i],os);
  }
}

